# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2022/2/17 2:45 下午
# Copyright (C) 2022 The lesscode Team
from lesscode.db.base_connection_pool import BaseConnectionPool
from elasticsearch import AsyncElasticsearch
from elasticsearch import Elasticsearch


class ElasticsearchPool(BaseConnectionPool):
    """
    Elasticsearch 数据库链接创建类
    """

    async def create_pool(self):
        """
        创建elasticsearch 异步连接池
        :param conn_info: 连接信息
        :return:
        """
        info = self.conn_info
        if info.async_enable:
            host_str = info.host.split(",")
            hosts = [f"http://{info.user}:{info.password}@{host}:{info.port}" for host in host_str]
            pool = AsyncElasticsearch(hosts=hosts)
            return pool
        else:
            raise NotImplementedError

    def sync_create_pool(self):
        info = self.conn_info
        host_str = info.host.split(",")
        hosts = [f"http://{info.user}:{info.password}@{host}:{info.port}" for host in host_str]
        pool = Elasticsearch(hosts)
        return pool



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2021/12/8 10:53 上午
# Copyright (C) 2021 The lesscode Team
import json
import logging
import os
import random
from datetime import datetime

import requests
from requests.auth import HTTPBasicAuth
from tornado.options import options

from lesscode.db.base_sql_helper import BaseSqlHelper, echo_sql
from lesscode.db.condition_wrapper import ConditionWrapper
from lesscode.db.connection_info import ConnectionInfo
from lesscode.db.es.es_pool import EsPool
from lesscode.db.page import Page
from lesscode.utils.EsUtil import format_es_param_result
from lesscode.utils.encryption_algorithm import AES


class EsHelper:
    """
    ElasticsearchHelper  ES数据库操作实现
    """

    def __init__(self, pool):
        """
        初始化sql工具
        :param pool: 连接池名称
        """
        if isinstance(pool, str):
            self.pool, self.dialect = options.database[pool]
        else:
            self.pool = pool

    async def send_es_post(self, bool_must_list=None, param_list=None, route_key="", sort_list=None, size=10,
                           offset=0,
                           track_total_hits=False):
        params = {
            "query": {
                "bool": {
                    "must": bool_must_list
                }
            },
            "size": size,
            "from": offset,
        }
        if track_total_hits:
            params["track_total_hits"] = track_total_hits
        if param_list:
            params["_source"] = {"include": param_list}
        if sort_list:
            params["sort"] = sort_list
        start_time = datetime.now()

        res = self.pool.es_selector_way(url_func_str=self.pool.format_es_post_url, param_dict={
            "route_key": route_key,
        }, find_condition=params)
        logging.info("进程{}，路由{},查询时间{}".format(os.getpid(), route_key, datetime.now() - start_time))

        if "error" in list(res.keys()):
            logging.info(res)
        return res["hits"]

    async def format_es_return(self, bool_must_list=None, param_list=None, route_key="", sort_list=None, size=10,
                               offset=0,
                               track_total_hits=False, is_need_es_score=False, is_need_decrypt_oralce=False, res=None):
        if not res:
            res = await self.send_es_post(bool_must_list, param_list, route_key=route_key, sort_list=sort_list,
                                          size=size,
                                          offset=offset,
                                          track_total_hits=track_total_hits)

        result_list = []
        for r in res["hits"]:
            result_list.append(
                format_es_param_result(r, param_list, is_need_decrypt_oralce, is_need_es_score, route_key))
        result_dict = {
            "data_count": res["total"]["value"],
            "data_list": result_list
        }
        return result_dict

    async def format_es_scan(self, bool_must_list=None, param_list=None, route_key="", scroll="5m", size=10000,
                             is_need_decrypt_oralce=False, limit=None):
        logging.info("扫描开始，条件是{},查询字段是{}".format(json.dumps(bool_must_list), json.dumps(param_list)))
        skip = 0
        request_param = {
            "query": {
                "bool": {
                    "must": bool_must_list
                }
            }
            , "size": size,
        }
        if param_list:
            request_param["_source"] = {"include": param_list}
        res = self.pool.es_selector_way(url_func_str=self.pool.format_scroll_url, param_dict={
            "route_key": route_key,
            "scroll": scroll
        }, find_condition=request_param)
        data_size = len(res["hits"]["hits"])
        logging.info(
            "扫描{}:{}条花费时间{}ms,".format(route_key, str(skip) + "-" + str(skip + data_size), res["took"]))
        scroll_id = res["_scroll_id"]
        result_list = []
        for data in res["hits"]["hits"]:
            if is_need_decrypt_oralce:
                data["_id"] = AES.encrypt(options.aes_key, data["_id"])
            data["_source"]["_id"] = data["_id"]
            result_list.append(data["_source"])
        while True:
            skip = skip + data_size

            res = self.pool.es_selector_way(url_func_str=self.pool.format_scroll_id_url, param_dict={
            }, find_condition={
                "scroll": scroll,
                "scroll_id": scroll_id})
            data_size = len(res["hits"]["hits"])
            logging.info("扫描{}:{}条花费时间{}ms,".format(route_key, str(skip) + "-" + str(skip + data_size), res["took"]))
            scroll_id = res.get("_scroll_id")
            # end of scroll
            if scroll_id is None or not res["hits"]["hits"]:
                break
            for data in res["hits"]["hits"]:
                data["_source"]["_id"] = data["_id"]
                result_list.append(data["_source"])
            if limit and limit <= len(result_list):
                break
        return result_list

    async def format_es_group(self, bool_must_list=None, route_key="", aggs=None):
        params = {
            "query": {
                "bool": {
                    "must": bool_must_list
                }
            },
            "size": 0,
            "aggs": aggs
        }
        res = self.pool.es_selector_way(url_func_str=self.pool.format_es_post_url, param_dict={
            "route_key": route_key,
        }, find_condition=params)
        return res

    def sync_send_es_post(self, bool_must_list=None, param_list=None, route_key="", sort_list=None, size=10,
                          offset=0,
                          track_total_hits=False):
        params = {
            "query": {
                "bool": {
                    "must": bool_must_list
                }
            },
            "size": size,
            "from": offset,
        }
        if track_total_hits:
            params["track_total_hits"] = track_total_hits
        if param_list:
            params["_source"] = {"include": param_list}
        if sort_list:
            params["sort"] = sort_list
        start_time = datetime.now()

        res = self.pool.es_selector_way(url_func_str=self.pool.format_es_post_url, param_dict={
            "route_key": route_key,
        }, find_condition=params)
        logging.info("进程{}，路由{},查询时间{}".format(os.getpid(), route_key, datetime.now() - start_time))

        if "error" in list(res.keys()):
            logging.info(res)
        return res["hits"]

    def sync_format_es_return(self, bool_must_list=None, param_list=None, route_key="", sort_list=None, size=10,
                              offset=0,
                              track_total_hits=False, is_need_es_score=False, is_need_decrypt_oralce=False, res=None):
        if not res:
            res = self.sync_send_es_post(bool_must_list, param_list, route_key=route_key, sort_list=sort_list,
                                         size=size,
                                         offset=offset,
                                         track_total_hits=track_total_hits)

        result_list = []
        for r in res["hits"]:
            result_list.append(
                format_es_param_result(r, param_list, is_need_decrypt_oralce, is_need_es_score, route_key))
        result_dict = {
            "data_count": res["total"]["value"],
            "data_list": result_list
        }
        return result_dict

    def sync_format_es_scan(self, bool_must_list=None, param_list=None, route_key="", scroll="5m", size=10000,
                            is_need_decrypt_oralce=False, limit=None):
        logging.info("扫描开始，条件是{},查询字段是{}".format(json.dumps(bool_must_list), json.dumps(param_list)))
        skip = 0
        request_param = {
            "query": {
                "bool": {
                    "must": bool_must_list
                }
            }
            , "size": size,
        }
        if param_list:
            request_param["_source"] = {"include": param_list}
        res = self.pool.es_selector_way(url_func_str=self.pool.format_scroll_url, param_dict={
            "route_key": route_key,
            "scroll": scroll
        }, find_condition=request_param)
        data_size = len(res["hits"]["hits"])
        logging.info(
            "扫描{}:{}条花费时间{}ms,".format(route_key, str(skip) + "-" + str(skip + data_size), res["took"]))
        scroll_id = res["_scroll_id"]
        result_list = []
        for data in res["hits"]["hits"]:
            if is_need_decrypt_oralce:
                data["_id"] = AES.encrypt(options.aes_key, data["_id"])
            data["_source"]["_id"] = data["_id"]
            result_list.append(data["_source"])
        while True:
            skip = skip + data_size

            res = self.pool.es_selector_way(url_func_str=self.pool.format_scroll_id_url, param_dict={
            }, find_condition={
                "scroll": scroll,
                "scroll_id": scroll_id})
            data_size = len(res["hits"]["hits"])
            logging.info("扫描{}:{}条花费时间{}ms,".format(route_key, str(skip) + "-" + str(skip + data_size), res["took"]))
            scroll_id = res.get("_scroll_id")
            # end of scroll
            if scroll_id is None or not res["hits"]["hits"]:
                break
            for data in res["hits"]["hits"]:
                data["_source"]["_id"] = data["_id"]
                result_list.append(data["_source"])
            if limit and limit <= len(result_list):
                break
        return result_list

    def sync_format_es_group(self, bool_must_list=None, route_key="", aggs=None):
        params = {
            "query": {
                "bool": {
                    "must": bool_must_list
                }
            },
            "size": 0,
            "aggs": aggs
        }
        res = self.pool.es_selector_way(url_func_str=self.pool.format_es_post_url, param_dict={
            "route_key": route_key,
        }, find_condition=params)
        return res



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2021/11/24 7:34 下午
# Copyright (C) 2021 The lesscode Team

from tornado.options import options, define

from lesscode.web.business_exception import BusinessException
from lesscode.web.status_code import StatusCode

define("database", default={}, type=dict, help="数据库链接资源池,内部使用")
define("conn_info", default=None, type=list, help="数据库链接信息")


class InitConnectionPool:
    """
    初始化连接池
    """

    @staticmethod
    async def create_pool():
        """
        获取配置的链接信息，生成对应的连接池
        :return:
        """
        if options.conn_info:
            for conn_info in options.conn_info:
                if conn_info.enable:
                    if options.database.keys().__contains__(conn_info.name):
                        # 相同key的连接池已经存在，抛出异常
                        raise BusinessException(StatusCode.RESOURCE_EXIST(f"{conn_info.name}连接池"))
                    pool = None
                    # 依据数据库类型进行链接池创建
                    if conn_info.dialect == "postgresql":
                        clazz = InitConnectionPool.getClass("lesscode.db.postgresql.postgresql_pool",
                                                            fromlist="PostgresqlPool")
                        # 执行用例
                        pool = await clazz(conn_info).create_pool()
                    elif conn_info.dialect == "mysql":
                        # 获取包中用例的函数引用
                        clazz = InitConnectionPool.getClass("lesscode.db.mysql.mysql_pool", fromlist="MysqlPool")
                        # 执行用例
                        pool = await clazz(conn_info).create_pool()
                    elif conn_info.dialect == "mongodb":
                        # 获取包中用例的函数引用
                        clazz = InitConnectionPool.getClass("lesscode.db.mongodb.mongodb_pool", fromlist="MongodbPool")
                        # 执行用例
                        pool = clazz(conn_info).create_pool()
                    elif conn_info.dialect == "elasticsearch":
                        # 获取包中用例的函数引用
                        clazz = InitConnectionPool.getClass("lesscode.db.elasticsearch.elasticsearch_pool",
                                                            fromlist="ElasticsearchPool")
                        # 执行用例
                        pool = await clazz(conn_info).create_pool()
                    elif conn_info.dialect == "esapi":
                        # 获取包中用例的函数引用
                        clazz = InitConnectionPool.getClass("lesscode.db.es.es_pool", fromlist="EsPool")
                        # 执行用例
                        pool = clazz(conn_info).create_pool()
                    elif conn_info.dialect == "redis":
                        # 获取包中用例的函数引用
                        clazz = InitConnectionPool.getClass("lesscode.db.redis.redis_pool", fromlist="RedisPool")
                        # 执行用例
                        pool = clazz(conn_info).create_pool()
                    elif conn_info.dialect == "neo4j":
                        # 获取包中用例的函数引用
                        clazz = InitConnectionPool.getClass("lesscode.db.neo4j.neo4j_pool", fromlist="Neo4jPool")
                        # 执行用例
                        pool = await clazz(conn_info).create_pool()
                    elif conn_info.dialect == "mysql_sqlalchemy":
                        clazz = InitConnectionPool.getClass("lesscode.db.mysql_sqlalchemy.mysql_sqlalchemy_pool",
                                                            fromlist="MysqlSqlAlchemyPool")
                        # 执行用例
                        pool = clazz(conn_info).sync_create_pool()
                    if pool:
                        options.database[conn_info.name] = (pool, conn_info.dialect)

    @staticmethod
    def sync_create_pool():
        """
                获取配置的链接信息，生成对应的连接池
                :return:
                """
        if options.conn_info:
            for conn_info in options.conn_info:
                if conn_info.enable:
                    if options.database.keys().__contains__(conn_info.name):
                        # 相同key的连接池已经存在，抛出异常
                        raise BusinessException(StatusCode.RESOURCE_EXIST(f"{conn_info.name}连接池"))
                    pool = None
                    # 依据数据库类型进行链接池创建
                    if conn_info.dialect == "postgresql":
                        clazz = InitConnectionPool.getClass("lesscode.db.postgresql.postgresql_pool",
                                                            fromlist="PostgresqlPool")
                        # 执行用例
                        pool = clazz(conn_info).sync_create_pool()
                    elif conn_info.dialect == "mysql":
                        # 获取包中用例的函数引用
                        clazz = InitConnectionPool.getClass("lesscode.db.mysql.mysql_pool", fromlist="MysqlPool")
                        # 执行用例
                        pool = clazz(conn_info).sync_create_pool()
                    elif conn_info.dialect == "mongodb":
                        # 获取包中用例的函数引用
                        clazz = InitConnectionPool.getClass("lesscode.db.mongodb.mongodb_pool", fromlist="MongodbPool")
                        # 执行用例
                        pool = clazz(conn_info).sync_create_pool()
                    elif conn_info.dialect == "elasticsearch":
                        # 获取包中用例的函数引用
                        clazz = InitConnectionPool.getClass("lesscode.db.elasticsearch.elasticsearch_pool",
                                                            fromlist="ElasticsearchPool")
                        # 执行用例
                        pool = clazz(conn_info).sync_create_pool()
                    elif conn_info.dialect == "esapi":
                        # 获取包中用例的函数引用
                        clazz = InitConnectionPool.getClass("lesscode.db.es.es_pool", fromlist="EsPool")
                        # 执行用例
                        pool = clazz(conn_info).create_pool()
                    elif conn_info.dialect == "redis":
                        # 获取包中用例的函数引用
                        clazz = InitConnectionPool.getClass("lesscode.db.redis.redis_pool", fromlist="RedisPool")
                        # 执行用例
                        pool = clazz(conn_info).sync_create_pool()
                    elif conn_info.dialect == "neo4j":
                        # 获取包中用例的函数引用
                        clazz = InitConnectionPool.getClass("lesscode.db.neo4j.neo4j_pool", fromlist="Neo4jPool")
                        # 执行用例
                        pool = clazz(conn_info).sync_create_pool()
                    elif conn_info.dialect == "mysql_sqlalchemy":
                        clazz = InitConnectionPool.getClass("lesscode.db.mysql_sqlalchemy.mysql_sqlalchemy_pool",
                                                            fromlist="MysqlSqlAlchemyPool")
                        # 执行用例
                        pool = clazz(conn_info).sync_create_pool()
                    if pool:
                        options.database[conn_info.name] = (pool, conn_info.dialect)

    @staticmethod
    def getClass(name, fromlist):
        return getattr(__import__(name, fromlist=fromlist), fromlist)
    # @staticmethod
    # async def create_pool_postgresql(conn_info: ConnectionInfo):
    #     """
    #     创建postgresql 异步连接池
    #     :param conn_info: 连接信息
    #     :return: AsyncConnectionPool
    #     """
    #     if conn_info.async_enable:
    #         from aiopg import create_pool
    #         pool = await create_pool(host=conn_info.host, port=conn_info.port, user=conn_info.user,
    #                                  password=conn_info.password,
    #                                  database=conn_info.db_name)
    #         return pool
    #     else:
    #         raise NotImplementedError

    # @staticmethod
    # async def create_pool_mysql(conn_info: ConnectionInfo):
    #     """
    #     创建mysql 异步连接池
    #     :param conn_info: 连接信息
    #     :return:
    #     """
    #     if conn_info.async_enable:
    #         import aiomysql
    #         pool = await aiomysql.create_pool(host=conn_info.host, port=conn_info.port, user=conn_info.user,
    #                                           password=conn_info.password, pool_recycle=60,
    #                                           db=conn_info.db_name, autocommit=True, minsize=conn_info.min_size,
    #                                           maxsize=conn_info.max_size)
    #         return pool
    #     else:
    #         raise NotImplementedError

    # @staticmethod
    # def create_pool_mongodb(conn_info: ConnectionInfo):
    #     """
    #     创建mongodb 异步连接池
    #     :param conn_info: 连接信息
    #     :return:
    #     """
    #     if conn_info.async_enable:
    #         import motor
    #         conn_info_string = f"mongodb://{conn_info.user}:{conn_info.password}@{conn_info.host}:{conn_info.port}"
    #         if conn_info.params:
    #             if conn_info.params == "LDAP":
    #                 conn_info_string += "/?authMechanism=PLAIN"
    #             elif conn_info.params == "Password":
    #                 conn_info_string += "/?authSource=admin"
    #             elif conn_info.params == "X509":
    #                 conn_info_string += "/?authMechanism=MONGODB-X509"
    #         pool = motor.motor_tornado.MotorClient(conn_info_string)
    #         return pool
    #     else:
    #         raise NotImplementedError

    # @staticmethod
    # async def create_pool_es(conn_info: ConnectionInfo):
    #     """
    #     创建elasticsearch 异步连接池
    #     :param conn_info: 连接信息
    #     :return:
    #     """
    #     if conn_info.async_enable:
    #         from elasticsearch import AsyncElasticsearch
    #         host_str = conn_info.host.split(",")
    #         hosts = [f"{conn_info.user}:{conn_info.password}@{host}:{conn_info.port}" for host in host_str]
    #         pool = AsyncElasticsearch(hosts=hosts)
    #         return pool
    #     else:
    #         raise NotImplementedError



import base64

from Crypto.Cipher import DES as Des


class DES:
    @staticmethod
    def encrypt(string: str, key: str, mode=Des.MODE_ECB, encoding='utf-8'):
        pad = 8 - len(string) % 8
        pad_str = "".join([chr(i) for i in range(pad)])
        string += pad_str
        des = Des.new(key=key, mode=mode)
        encrypt_string = base64.b64encode(des.encrypt(string.encode(encoding))).decode(encoding)
        return encrypt_string

    @staticmethod
    def decrypt(string: str, key: str, mode=Des.MODE_ECB, encoding='utf-8'):
        pad = 8 - len(string) % 8
        pad_str = "".join([chr(i) for i in range(pad)])
        string += pad_str
        des = Des.new(key=key, mode=mode)
        decrypt_string = des.decrypt(base64.b64decode(string.encode(encoding))).decode(encoding)
        return decrypt_string



from .aes import AES
from .md5 import MD5
from .sha256 import SHA256
from .sha1 import SHA1
from .base64 import BASE64
from .rsa import RSA
from .hmac import HMAC
from .des import DES
from .smx import SMX
from .des import DES



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2022/2/17 2:45 下午
# Copyright (C) 2022 The lesscode Team
import asyncio

import pymysql
from dbutils.pooled_db import PooledDB
from tornado.options import options, define
from tornado_sqlalchemy import SQLAlchemy

from lesscode.db.base_connection_pool import BaseConnectionPool
import aiomysql




class MysqlSqlAlchemyPool(BaseConnectionPool):
    """
    mysql 数据库链接创建类
    """

    def sync_create_pool(self):
        mysql_url = 'mysql+pymysql://{}:{}@{}:{}/{}?charset=utf8'.format(
            self.conn_info.user, self.conn_info.password, self.conn_info.host, self.conn_info.port,
            self.conn_info.db_name)
        db = SQLAlchemy(mysql_url, engine_options={"echo": True, "pool_recycle": 3600})
        options.db = db
        return db



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2021/11/26 3:56 下午
# Copyright (C) 2021 The lesscode Team
import asyncio
from asyncio import events

from tornado.options import options

from lesscode.db.base_sql_helper import BaseSqlHelper
from lesscode.db.condition_wrapper import ConditionWrapper


class DSHelper(BaseSqlHelper):
    """
    对外暴露的工具类，对数据库操作采用代理模式进行了包装
    """

    def __init__(self, pool_name):
        """
        初始化sql工具
        :param pool_name: 连接池名称
        """
        self.pool, self.dialect = options.database[pool_name]
        if self.dialect == "postgresql":
            # self.db_helper = PostgresqlHelper(self.pool)
            clazz = getattr(__import__("lesscode.db.postgresql.postgresql_helper", fromlist="PostgresqlHelper"),
                            "PostgresqlHelper")
        elif self.dialect == "mysql":
            # self.db_helper = MysqlHelper(self.pool)
            clazz = getattr(__import__("lesscode.db.mysql.mysql_helper", fromlist="MysqlHelper"), "MysqlHelper")
            # 执行用例
            self.db_helper = clazz(self.pool)
        elif self.dialect == "mongodb":
            clazz = getattr(__import__("lesscode.db.mongodb.mongodb_helper", fromlist="MongodbHelper"), "MongodbHelper")
            # self.db_helper = MongodbHelper(self.pool)
        elif self.dialect == "elasticsearch":
            clazz = getattr(
                __import__("lesscode.db.elasticsearch.elasticsearch_helper", fromlist="ElasticsearchHelper"),
                "ElasticsearchHelper")
        elif self.dialect == "esapi":
            clazz = getattr(__import__("lesscode.db.es.es_helper", fromlist="EsHelper"), "EsHelper")
        elif self.dialect == "redis":
            clazz = getattr(
                __import__("lesscode.db.redis.redis_helper", fromlist="RedisHelper"),
                "RedisHelper")
        if clazz:
            self.db_helper = clazz(self.pool)

    async def insert_data(self, table_name: str, data):
        return await self.db_helper.insert_data(table_name, data)

    async def insert_one_data(self, table_name: str, data: dict):
        return await self.db_helper.insert_one_data(table_name, data)

    async def insert_many_data(self, table_name: str, data: list):
        return await self.db_helper.insert_many_data(table_name, data)

    async def update_data(self, condition_wrapper: ConditionWrapper, param: dict):
        return await self.db_helper.update_data(condition_wrapper, param)

    async def delete_data(self, condition_wrapper: ConditionWrapper):
        return await self.db_helper.delete_data(condition_wrapper)

    async def fetchone_data(self, condition_wrapper: ConditionWrapper):
        return await self.db_helper.fetchone_data(condition_wrapper)

    async def fetchall_data(self, condition_wrapper: ConditionWrapper):
        return await self.db_helper.fetchall_data(condition_wrapper)

    async def fetchall_page(self, condition_wrapper: ConditionWrapper, page_num=1, page_size=10):
        return await self.db_helper.fetchall_page(condition_wrapper, page_num, page_size)

    async def execute_sql(self, sql: str, param=None):
        return await self.db_helper.execute_sql(sql, param)

    async def executemany_sql(self, sql: str, param=None):
        return await self.db_helper.executemany_sql(sql, param)

    async def execute_fetchone(self, sql: str, param=None):
        return await self.db_helper.execute_fetchone(sql, param)

    async def execute_fetchall(self, sql: str, param=None):
        return await self.db_helper.execute_fetchall(sql, param)

    def prepare_insert_sql(self, table_name: str, item: dict):
        return self.db_helper.prepare_insert_sql(table_name, item)

    def prepare_update_sql(self, condition_wrapper: ConditionWrapper, param: dict):
        return self.db_helper.prepare_update_sql(condition_wrapper, param)

    def prepare_delete_sql(self, condition_wrapper: ConditionWrapper):
        return self.db_helper.prepare_delete_sql(condition_wrapper)

    def prepare_condition_sql(self, conditions: list):
        return self.db_helper.prepare_condition_sql(conditions)

    def prepare_query_sql(self, condition_wrapper: ConditionWrapper):
        return self.db_helper.prepare_query_sql(condition_wrapper)

    def prepare_page_sql(self, condition_wrapper: ConditionWrapper, page_num: int, page_size: int):
        return self.db_helper.prepare_page_sql(condition_wrapper, page_num, page_size)

    def sync_insert_data(self, table_name: str, data):
        return self.db_helper.sync_insert_data(table_name, data)

    def sync_insert_one_data(self, table_name: str, data: dict):
        return self.db_helper.sync_insert_one_data(table_name, data)

    def sync_insert_many_data(self, table_name: str, data: list):
        return self.db_helper.sync_insert_many_data(table_name, data)

    def sync_update_data(self, condition_wrapper: ConditionWrapper, param: dict):
        return self.db_helper.sync_update_data(condition_wrapper, param)

    def sync_delete_data(self, condition_wrapper: ConditionWrapper):
        return self.db_helper.sync_delete_data(condition_wrapper)

    def sync_fetchone_data(self, condition_wrapper: ConditionWrapper):
        return self.db_helper.sync_fetchone_data(condition_wrapper)

    def sync_fetchall_data(self, condition_wrapper: ConditionWrapper):
        return self.db_helper.sync_fetchall_data(condition_wrapper)

    def sync_fetchall_page(self, condition_wrapper: ConditionWrapper, page_num=1, page_size=10):
        return self.db_helper.sync_fetchall_page(condition_wrapper, page_num, page_size)

    def sync_execute_sql(self, sql: str, param=None):
        return self.db_helper.sync_execute_sql(sql, param)

    def sync_executemany_sql(self, sql: str, param=None):
        return self.db_helper.sync_executemany_sql(sql, param)

    def sync_execute_fetchone(self, sql: str, param=None):
        return self.db_helper.sync_execute_fetchone(sql, param)

    def sync_execute_fetchall(self, sql: str, param=None):
        return self.db_helper.sync_execute_fetchall(sql, param)



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2022/2/17 2:45 下午
# Copyright (C) 2022 The lesscode Team
import neo4j
from neo4j import GraphDatabase, AsyncGraphDatabase

from lesscode.db.base_connection_pool import BaseConnectionPool


class Neo4jPool(BaseConnectionPool):
    """
    Neo4j 数据库链接创建类
    """

    async def create_pool(self):
        """
        创建Neo4j 连接池
        :return:
        """
        driver = AsyncGraphDatabase.driver(f"bolt://{self.conn_info.host}:{self.conn_info.port}",
                                           auth=(self.conn_info.user, self.conn_info.password))
        access_mode = self.conn_info.params.get("access_mode", "READ") if self.conn_info.params else "READ"
        if access_mode == "WRITE":
            access = neo4j.WRITE_ACCESS
        else:
            access = neo4j.READ_ACCESS
        pool = driver.session(database=self.conn_info.db_name, default_access_mode=access)
        return pool

    def sync_create_pool(self):
        """
        创建Neo4j 连接池
        :return:
        """
        driver = GraphDatabase.driver(f"bolt://{self.conn_info.host}:{self.conn_info.port}",
                                      auth=(self.conn_info.user, self.conn_info.password))
        access_mode = self.conn_info.params.get("access_mode", "READ") if self.conn_info.params else "READ"
        if access_mode == "WRITE":
            access = neo4j.WRITE_ACCESS
        else:
            access = neo4j.READ_ACCESS
        pool = driver.session(database=self.conn_info.db_name, default_access_mode=access)
        return pool



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2022/2/17 2:45 下午
# Copyright (C) 2022 The lesscode Team
import asyncio

import pymysql
from dbutils.pooled_db import PooledDB

from lesscode.db.base_connection_pool import BaseConnectionPool
import aiomysql


class MysqlPool(BaseConnectionPool):
    """
    mysql 数据库链接创建类
    """

    async def create_pool(self):
        """
        创建mysql 异步连接池
        :return:
        """
        if self.conn_info.async_enable:
            pool = await aiomysql.create_pool(host=self.conn_info.host, port=self.conn_info.port,
                                              user=self.conn_info.user,
                                              password=self.conn_info.password,
                                              pool_recycle=self.conn_info.params.get("pool_recycle",
                                                                                     3600 * 4)
                                              if self.conn_info.params else 3600 * 4,
                                              db=self.conn_info.db_name, autocommit=True,
                                              minsize=self.conn_info.min_size,
                                              maxsize=self.conn_info.max_size)
            loop = asyncio.get_running_loop()
            loop.create_task(self.keep_connect(pool))
            return pool
        else:
            raise NotImplementedError

    async def keep_connect(self, pool):
        while True:
            async with pool.acquire() as conn:
                await conn.ping()
            await asyncio.sleep(60)

    def sync_create_pool(self):
        pool = PooledDB(pymysql, 5, host=self.conn_info.host, port=self.conn_info.port, user=self.conn_info.user,
                        passwd=self.conn_info.password, db=self.conn_info.db_name, charset="utf8", autocommit=True)
        return pool



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2021/12/3 2:19 下午
# Copyright (C) 2021 The lesscode Team
import logging
from tornado.options import options
from lesscode.db.base_sql_helper import BaseSqlHelper, echo_sql
from lesscode.db.condition_wrapper import ConditionWrapper
from lesscode.db.page import Page


class MongodbHelper(BaseSqlHelper):
    """
    MongodbHelper  Mongodb数据库操作实现
    """

    async def insert_data(self, table_name: str, data):
        """
        对于插入接口的整合，不需要考虑是单条插入还是多条插入，自动区分
        :param table_name:
        :param data:
        :return:
        """
        if isinstance(data, list):
            if len(data) == 1:
                return await self.insert_one_data(table_name, data[0])
            else:
                return await self.insert_many_data(table_name, data)
        elif isinstance(data, dict):
            return await self.insert_one_data(table_name, data)

    async def insert_one_data(self, table_name: str, data: dict):
        """
        新增记录插入
        :param table_name: 表名
        :param data: 待插入数据
        :return:
        """
        db_str, collection_str = table_name.split(".")
        db = self.pool[db_str]
        collection = db[collection_str]
        await collection.insert_one(data)
        return {"rowcount": 1}

    async def insert_many_data(self, table_name: str, data: list):
        """
        批量插入
        :param table_name:
        :param data:
        :return:
        """
        db_str, collection_str = table_name.split(".")
        db = self.pool[db_str]
        collection = db[collection_str]
        rs = await collection.insert_many(data)
        return {"rowcount": len(rs.inserted_ids)}

    async def update_data(self, condition_wrapper: ConditionWrapper, param: dict):
        """
        按条件更新
        :param condition_wrapper: 更新条件
        :param param:更新值
        :return:
        """
        db_str, collection_str = condition_wrapper.table.split(".")
        db = self.pool[db_str]
        collection = db[collection_str]
        filter_items = self.prepare_condition_sql(condition_wrapper.conditions)
        rs = await collection.update_many(filter_items, {"$set": param})
        return {"rowcount": rs.modified_count}

    async def delete_data(self, condition_wrapper: ConditionWrapper):
        """
        按条件删除
        :param condition_wrapper: 删除条件
        :return:
        """
        db_str, collection_str = condition_wrapper.table.split(".")
        db = self.pool[db_str]
        collection = db[collection_str]
        filter_items = self.prepare_condition_sql(condition_wrapper.conditions)
        rs = await collection.delete_many(filter_items)
        return {"rowcount": rs.deleted_count}

    async def fetchone_data(self, condition_wrapper: ConditionWrapper):
        """
        查询单条数据
        :param condition_wrapper:  查询条件信息
        :return:
        """
        db_str, collection_str = condition_wrapper.table.split(".")
        db = self.pool[db_str]
        collection = db[collection_str]
        aggregate = self.prepare_query_sql(condition_wrapper)
        return await self.execute_fetchone(aggregate, {"collection": collection})

    async def fetchall_data(self, condition_wrapper: ConditionWrapper):
        """
        查询多条数据
        :param condition_wrapper: 查询条件信息
        :return:
        """
        db_str, collection_str = condition_wrapper.table.split(".")
        db = self.pool[db_str]
        collection = db[collection_str]
        aggregate = self.prepare_query_sql(condition_wrapper)
        rs = await self.execute_fetchall(aggregate, {"collection": collection, "length": options.max_limit})
        return rs

    async def fetchall_page(self, condition_wrapper: ConditionWrapper, page_num=1, page_size=10):
        """
        分页查询多条数据
        :param condition_wrapper:
        :param page_num: 当前页码
        :param page_size: 每页数量
        :return:
        """
        db_str, collection_str = condition_wrapper.table.split(".")
        db = self.pool[db_str]
        collection = db[collection_str]
        aggregate, count_ = self.prepare_page_sql(condition_wrapper, page_num, page_size)
        records = await self.execute_fetchall(aggregate, {"collection": collection, "length": int(page_size)})
        # 查询记录数量
        total_result = await self.execute_fetchone(count_, {"collection": collection})
        # 组装成page包装类 最终转为 dict 类型 为转换json
        return Page(records=records, current=page_num, page_size=page_size,
                    total=total_result.get("total", 0) if total_result else 0).__dict__

    @echo_sql
    async def execute_fetchone(self, sql: str, param=None):
        """
        查询单条数据
        :param sql: 待执行的Sql语句
        :param param: 参数
        :return:
        """
        cursor = param["collection"].aggregate(sql)
        rs = await cursor.to_list(1)
        if len(rs) > 0:
            return rs[0]
        return None

    @echo_sql
    async def execute_fetchall(self, sql: str, param=None):
        """
        查询多条数据
        :param sql: 待执行的Sql语句
        :param param: 参数
        :return:
        """
        cursor = param["collection"].aggregate(sql)
        return await cursor.to_list(length=param["length"])

    @echo_sql
    async def execute_fetch(self, db_collection, sql: str, length=10000):
        """
        查询多条数据
        :param sql: 待执行的Sql语句
        :param param: 参数
        :return:
        """
        db_str, collection_str = db_collection.split(".")
        db = self.pool[db_str]
        collection = db[collection_str]
        cursor = collection.aggregate(sql)
        return await cursor.to_list(length)

    @echo_sql
    async def execute_fetch_one(self, db_collection, sql):
        """
        查询单条数据
        :param sql: 待执行的Sql语句
        :param param: 参数
        :return:
        """
        db_str, collection_str = db_collection.split(".")
        db = self.pool[db_str]
        collection = db[collection_str]
        # collection.find_one(find_condition)
        cursor = collection.aggregate(sql)
        rs = await cursor.to_list(1)
        if len(rs) > 0:
            return rs[0]
        return None

    def sync_insert_data(self, table_name: str, data):
        """
        对于插入接口的整合，不需要考虑是单条插入还是多条插入，自动区分
        :param table_name:
        :param data:
        :return:
        """
        if isinstance(data, list):
            if len(data) == 1:
                return self.insert_one_data(table_name, data[0])
            else:
                return self.sync_insert_many_data(table_name, data)
        elif isinstance(data, dict):
            return self.insert_one_data(table_name, data)

    def sync_insert_one_data(self, table_name: str, data: dict):
        """
        新增记录插入
        :param table_name: 表名
        :param data: 待插入数据
        :return:
        """
        db_str, collection_str = table_name.split(".")
        db = self.pool[db_str]
        collection = db[collection_str]
        collection.insert_one(data)
        return {"rowcount": 1}

    def sync_insert_many_data(self, table_name: str, data: list):
        """
        批量插入
        :param table_name:
        :param data:
        :return:
        """
        db_str, collection_str = table_name.split(".")
        db = self.pool[db_str]
        collection = db[collection_str]
        rs = collection.insert_many(data)
        return {"rowcount": len(rs.inserted_ids)}

    def sync_update_data(self, condition_wrapper: ConditionWrapper, param: dict):
        """
        按条件更新
        :param condition_wrapper: 更新条件
        :param param:更新值
        :return:
        """
        db_str, collection_str = condition_wrapper.table.split(".")
        db = self.pool[db_str]
        collection = db[collection_str]
        filter_items = self.prepare_condition_sql(condition_wrapper.conditions)
        rs = collection.update_many(filter_items, {"$set": param})
        return {"rowcount": rs.modified_count}

    def sync_delete_data(self, condition_wrapper: ConditionWrapper):
        """
        按条件删除
        :param condition_wrapper: 删除条件
        :return:
        """
        db_str, collection_str = condition_wrapper.table.split(".")
        db = self.pool[db_str]
        collection = db[collection_str]
        filter_items = self.prepare_condition_sql(condition_wrapper.conditions)
        rs = collection.delete_many(filter_items)
        return {"rowcount": rs.deleted_count}

    def sync_fetchone_data(self, condition_wrapper: ConditionWrapper):
        """
        查询单条数据
        :param condition_wrapper:  查询条件信息
        :return:
        """
        db_str, collection_str = condition_wrapper.table.split(".")
        db = self.pool[db_str]
        collection = db[collection_str]
        aggregate = self.prepare_query_sql(condition_wrapper)
        return self.sync_execute_fetchone(aggregate, {"collection": collection})

    def sync_fetchall_data(self, condition_wrapper: ConditionWrapper):
        """
        查询多条数据
        :param condition_wrapper: 查询条件信息
        :return:
        """
        db_str, collection_str = condition_wrapper.table.split(".")
        db = self.pool[db_str]
        collection = db[collection_str]
        aggregate = self.prepare_query_sql(condition_wrapper)
        rs = self.sync_execute_fetchall(aggregate, {"collection": collection, "length": options.max_limit})
        return rs

    def sync_fetchall_page(self, condition_wrapper: ConditionWrapper, page_num=1, page_size=10):
        """
        分页查询多条数据
        :param condition_wrapper:
        :param page_num: 当前页码
        :param page_size: 每页数量
        :return:
        """
        db_str, collection_str = condition_wrapper.table.split(".")
        db = self.pool[db_str]
        collection = db[collection_str]
        aggregate, count_ = self.prepare_page_sql(condition_wrapper, page_num, page_size)
        records = self.sync_execute_fetchall(aggregate, {"collection": collection, "length": int(page_size)})
        # 查询记录数量
        total_result = self.sync_execute_fetchone(count_, {"collection": collection})
        # 组装成page包装类 最终转为 dict 类型 为转换json
        return Page(records=records, current=page_num, page_size=page_size,
                    total=total_result.get("total", 0) if total_result else 0).__dict__

    @echo_sql
    def sync_execute_fetchone(self, sql: str, param=None):
        """
        查询单条数据
        :param sql: 待执行的Sql语句
        :param param: 参数
        :return:
        """
        cursor = param["collection"].aggregate(sql)
        rs = list(cursor)
        if len(rs) > 0:
            return rs[0]
        return None

    @echo_sql
    def sync_execute_fetchall(self, sql: str, param=None):
        """
        查询多条数据
        :param sql: 待执行的Sql语句
        :param param: 参数
        :return:
        """
        cursor = param["collection"].aggregate(sql)
        return list(cursor)

    @echo_sql
    def sync_execute_fetch(self, db_collection, sql: str, length=10000):
        """
        查询多条数据
        :param sql: 待执行的Sql语句
        :param param: 参数
        :return:
        """
        db_str, collection_str = db_collection.split(".")
        db = self.pool[db_str]
        collection = db[collection_str]
        cursor = collection.aggregate(sql)
        return list(cursor)

    @echo_sql
    def sync_execute_fetch_one(self, db_collection, sql):
        """
        查询单条数据
        :param sql: 待执行的Sql语句
        :param param: 参数
        :return:
        """
        db_str, collection_str = db_collection.split(".")
        db = self.pool[db_str]
        collection = db[collection_str]
        # collection.find_one(find_condition)
        cursor = collection.aggregate(sql)
        rs = list(cursor)
        if len(rs) > 0:
            return rs[0]
        return None

    def prepare_insert_sql(self, table_name: str, item: dict):
        """
        组装插入sql
        :param table_name:
        :param item:
        :return:
        """
        pass

    def prepare_update_sql(self, condition_wrapper: ConditionWrapper, param: dict):
        """
        组装更新sql
        :param condition_wrapper:
        :param param:
        :return:
        """
        pass

    def prepare_delete_sql(self, condition_wrapper: ConditionWrapper):
        """
        组装删除SQl
        :param condition_wrapper:
        :return:
        """
        condition_sql, values = self.prepare_condition_sql(condition_wrapper.conditions)
        delete_sql = f"DELETE FROM {condition_wrapper.table} WHERE {condition_sql}"
        return delete_sql, values

    def prepare_condition_sql(self, conditions: list):
        """
        拼接条件SQL 搜集参数信息
        :param conditions: 条件信息
        :return: 条件sql片段, values 对应参数
        """
        connector = "$and"
        if len(conditions) == 0:
            return {}
        filter_item = {"$and": []}
        for condition in conditions:
            # 获取当前过滤条件
            current_filter_items = filter_item.get(connector)
            operator, column, value = condition
            if operator == "OR" or operator == "AND":
                # 当前操作符与连接符不相同，要进行切换
                if connector != operator:
                    connector = "$and" if operator == "AND" else "$or"
                if len(current_filter_items) == 0:
                    # 当前没有过滤条件就重新进行生成，仅发生在第一个条件就直接切换为OR的情况
                    filter_item = {connector: []}
                else:
                    filter_item = {connector: [filter_item]}
                continue
            if operator == "=":
                condition_item = {column: value}
            elif operator == "<>":
                condition_item = {column: {"$ne": value}}
            elif operator == ">":
                condition_item = {column: {"$gt": value}}
            elif operator == ">=":
                condition_item = {column: {"$gte": value}}
            elif operator == "<":
                condition_item = {column: {"$lt": value}}
            elif operator == "<=":
                condition_item = {column: {"$lte": value}}
            elif operator == "BETWEEN":
                # 大于等于小数 小于 大数
                condition_item = {"$and": [{column: {"$gte": value[0]}}, {column: {"$lt": value[1]}}]}
            elif operator == "NOT_BETWEEN":
                # 小于小数  大于等于大数
                condition_item = {"$and": [{column: {"$lt": value[0]}}, {column: {"$gte": value[1]}}]}
            elif operator == "LIKE":
                condition_item = {column: {"$regex": f".*{value}.*", "$options": "$i"}}
            elif operator == "NOT_LIKE":
                condition_item = {column: {"$not": {"$regex": f".*{value}.*", "$options": "$i"}}}
            elif operator == "LIKE_LEFT":
                condition_item = {column: {"$regex": f"^{value}.*", "$options": "$i"}}
            elif operator == "LIKE_RIGHT":
                condition_item = {column: {"$regex": f".*{value}$", "$options": "$i"}}
            elif operator == "IS_NULL":
                condition_item = {column: None}
            elif operator == "IS_NOT_NULL":
                condition_item = {column: {"$ne": None}}
            elif operator == "IN":
                condition_item = {column: {"$in": value}}
            elif operator == "NOT_IN":
                condition_item = {column: {"$nin": value}}
            else:
                continue
            current_filter_items.append(condition_item)
        logging.debug(f"MongoDB Condition:{filter_item}")
        return filter_item

    def prepare_query_sql(self, condition_wrapper: ConditionWrapper):
        """
        获取完整拼接查询SQL
        :param condition_wrapper: 查询条件对象
        :return:查询对象，分组对象，排序对象
        """
        aggregate = []
        # 聚合参数
        if condition_wrapper.conditions:
            filter_items = self.prepare_condition_sql(condition_wrapper.conditions)
            aggregate.append({"$match": filter_items})
        # 拼接 分组SQL
        if condition_wrapper.groups:
            aggregate.append({"$group": condition_wrapper.groups})
        # 拼接 排序SQL
        if condition_wrapper.order:
            order_item = {}
            for order in condition_wrapper.order:
                column, op = order
                order_item.setdefault(column, 1 if op == "ASC" else -1)
            aggregate.append({"$sort": order_item})
        if condition_wrapper.column != "*" and condition_wrapper.column:
            aggregate.append({"$project": condition_wrapper.column})
        return aggregate

    def prepare_page_sql(self, condition_wrapper: ConditionWrapper, page_num: int, page_size: int):
        """
        获取分页SQL
        :param condition_wrapper: 查询参数
        :param page_num: 当前页
        :param page_size: 每页记录数
        :return:page_sql 分页,  统计总数
        """
        aggregate = self.prepare_query_sql(condition_wrapper)
        if len(aggregate) == 0:
            aggregate = []
        # 组装查询数量语句
        count_ = aggregate.copy()
        count_.append({"$count": 'total'})
        # 组装分页
        aggregate.append({"$skip": Page.skip(page_num, page_size)})
        aggregate.append({"$limit": int(page_size)})
        return aggregate, count_

    async def execute_sql(self, sql: str, param=None):
        pass

    async def executemany_sql(self, sql: str, param=None):
        pass

    def sync_execute_sql(self, sql: str, param=None):
        pass

    def sync_executemany_sql(self, sql: str, param=None):
        pass



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2022/2/17 2:45 下午
# Copyright (C) 2022 The lesscode Team

import aioredis
import redis

from lesscode.db.base_connection_pool import BaseConnectionPool


class RedisPool(BaseConnectionPool):
    """
    mysql 数据库链接创建类
    """

    def create_pool(self):
        """
        创建mysql 异步连接池
        :return:
        """
        if self.conn_info.async_enable:
            pool = aioredis.ConnectionPool.from_url(f"redis://{self.conn_info.host}", port=self.conn_info.port,
                                                    password=self.conn_info.password,
                                                    db=self.conn_info.db_name, encoding="utf-8", decode_responses=True)
            return pool
        else:
            raise NotImplementedError

    def sync_create_pool(self):
        pool = redis.ConnectionPool.from_url(f"redis://{self.conn_info.host}", port=self.conn_info.port,
                                             password=self.conn_info.password,
                                             db=self.conn_info.db_name, encoding="utf-8", decode_responses=True)
        return pool



import importlib


class Algorithm:

    @staticmethod
    def get_one_way_encryption_algorithm_list():
        """
        单向加密算法
        :return:
        """
        one_way_encryption_algorithm_list = ["MD5", "SHA256", "SHA1", "HMAC"]
        return one_way_encryption_algorithm_list

    @staticmethod
    def get_symmetric_encryption_algorithm_list():
        """
        对称加密算法
        :return:
        """
        symmetric_encryption_algorithm_list = ["AES", "BASE64", "DES"]
        return symmetric_encryption_algorithm_list

    @staticmethod
    def get_asymmetric_encryption_algorithm_list():
        """
        非对称加密算法
        :return:
        """
        asymmetric_encryption_algorithm_list = ["RSA"]
        return asymmetric_encryption_algorithm_list

    @staticmethod
    def get_all_encryption_algorithm():
        one_way_encryption_algorithm_list = Algorithm.get_one_way_encryption_algorithm_list()
        symmetric_encryption_algorithm_list = Algorithm.get_symmetric_encryption_algorithm_list()
        asymmetric_encryption_algorithm_list = Algorithm.get_asymmetric_encryption_algorithm_list()
        algorithm_list = []
        algorithm_list.extend(one_way_encryption_algorithm_list)
        algorithm_list.extend(symmetric_encryption_algorithm_list)
        algorithm_list.extend(asymmetric_encryption_algorithm_list)
        return list(set(algorithm_list))

    @staticmethod
    def encrypt(string: str, algorithm: str, params: dict = {}):
        all_encryption_algorithm = Algorithm.get_all_encryption_algorithm()
        if algorithm in all_encryption_algorithm:
            algorithm_lib = importlib.import_module(f"lesscode.utils.encryption_algorithm")
            if params:
                return algorithm_lib.__getattribute__(algorithm).encrypt(string, **params)
            else:
                return algorithm_lib.__getattribute__(algorithm).encrypt(string)
        else:
            raise Exception(f"{algorithm} is unsupported algorithm,supported algorithm have {all_encryption_algorithm}")

    @staticmethod
    def decrypt(string: str, algorithm: str, params: dict = {}):
        symmetric_encryption_algorithm_list = Algorithm.get_symmetric_encryption_algorithm_list()
        asymmetric_encryption_algorithm_list = Algorithm.get_asymmetric_encryption_algorithm_list()
        encryption_algorithm_list = []
        encryption_algorithm_list.extend(symmetric_encryption_algorithm_list)
        encryption_algorithm_list.extend(asymmetric_encryption_algorithm_list)
        encryption_algorithm_list = list(set(encryption_algorithm_list))
        if algorithm in encryption_algorithm_list:
            algorithm_lib = importlib.import_module(f"lesscode.utils.encryption_algorithm")
            if params:
                return algorithm_lib.__getattribute__(algorithm).decrypt(string, **params)
            else:
                return algorithm_lib.__getattribute__(algorithm).decrypt(string, **params)
        else:
            raise Exception(
                f"{algorithm} is unsupported algorithm,supported algorithm have {encryption_algorithm_list}")
